import numpy as np
import matplotlib.pyplot as plt

A = np.genfromtxt( "datos.csv", delimiter=',', dtype=float )

X = A[:,0:2]
y = A[:,2]
print( X.shape )
print( y.shape )

# El modelo de la máquina de vectores
# de soporte
# https://scikit-learn.org/stable/modules/svm.html#mathematical-formulation
#   mvs.coef_
#   mvs.intercept_

labels = np.array( [ [0,1], [0,2], [1,2] ] )

W = np.array( [[-0.23474325,  0.23143308],
               [ 0.0649192,   0.15029882],
               [ 0.13788711,  0.11393033]] )

vb = np.array( [-1.1726594,  0.08903536, 0.40300953] )

def d( k, x ) :
	v = W[k] @ x + vb[k]
	return v[0]

va = np.zeros( (3,) )

votes = np.zeros( (3,), dtype=int )

def decision( x ) :
	i = 0
	while i < 3 :
		va[i] = d(i, x )
		if va[i] >= 0 :
			votes[i] = labels[i,0]
		else :
			votes[i] = labels[i,1]

		i += 1
	vb = np.sort( votes )
	return vb[1]

# Prueba
vv = np.zeros( (2,1) )
vv[0,0] = -2
vv[1,0] = 10

# vv[0,0] = -8
# vv[1,0] = -5

# vx[0,0] = 4.0
# vx[1,0] = 2.5

vv[0,0] = 5.0
vv[1,0] = -8.0
v = decision( vv )
print( va, votes, v  )

# ----------------------------
fig, ax = plt.subplots()
ax.set_aspect( 1 )

# Los puntos
ax.scatter( X[:,0], X[:,1], c=y )

xi = np.linspace( -9,  7, 10 )
y1 = (-vb[0] - W[0,0]*xi)/W[0,1]
y2 = (-vb[1] - W[1,0]*xi)/W[1,1]
y3 = (-vb[2] - W[2,0]*xi)/W[2,1]
ax.plot( xi, y1 )
ax.plot( xi, y2 )
ax.plot( xi, y3 )


# Create grid values first.
n = 100
xi = np.linspace(-10, 8, n)
yi = np.linspace(-10, 15, n)
vx, vy = np.meshgrid( xi, yi )

zi = np.zeros( (n,n) )
i = 0
while i < n :
	vv[1,0] = yi[i]
	j = 0
	while j < n :
		vv[0,0] = xi[j]
		zi[i,j] = decision( vv ) 
		j += 1
	i += 1

# Linearly interpolate the data (x, y) on a grid defined by (xi, yi).

ax.set_xlim( (-10,8) )
ax.set_ylim( (-10,15) )

# Para poner la escala del color:
cntr = ax.contourf(vx, vy, zi )
fig.colorbar(cntr, ax=ax)

# Los puntos
ax.scatter( X[:,0], X[:,1], c=y )

# Los vectores de soporte en rojo
# ax.scatter( svec[:,0], svec[:,1], c='red' )

plt.show( )

